@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.36.2.10 2010/01/09 09:04:22 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.49 2009/12/16 07:19:15 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2010	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -58,6 +58,9 @@
 /* pid file name */
 #define DEFAULT_PID_FILE_NAME "/var/run/pgpool/pgpool.pid"
 
+/* status file name */
+#define STATUS_FILE_NAME "pgpool_status"
+
 typedef enum {
 	POOL_CONTINUE = 0,
 	POOL_IDLE,
@@ -161,6 +164,15 @@ typedef struct {
 	char *health_check_user;		/* PostgreSQL user name for health check */
 	char *failover_command;     /* execute command when failover happens */
 	char *failback_command;     /* execute command when failback happens */
+
+	/*
+	 * If true, trigger fail over when writing to the backend
+	 * communication socket fails. This is the same behavior of
+	 * pgpool-II 2.2.x or earlier. If set to false, pgpool will report
+	 * an error and disconnect the session.
+	 */
+	int	fail_over_on_backend_error;
+
 	char *recovery_user;		/* PostgreSQL user name for online recovery */
 	char *recovery_password;		/* PostgreSQL user password for online recovery */
 	char *recovery_1st_stage_command;   /* Online recovery command in 1st stage */
@@ -173,6 +185,7 @@ typedef struct {
 						   data consistency */
 	int ignore_leading_white_space;		/* ignore leading white spaces of each query */
  	int log_statement; /* 0:false, 1: true - logs all SQL statements */
+ 	int log_per_node_statement; /* 0:false, 1: true - logs per node detailed SQL statements */
 
 	int parallel_mode;	/* if non 0, run in parallel query mode */
 
@@ -495,7 +508,8 @@ extern int select_load_balancing_node(void);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend,
-									  int reset_request);
+									  int connection_reuse,
+									  int first_ready_for_query_received);
 
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
@@ -648,10 +662,12 @@ extern void finish_recovery(void);
 extern void pool_set_nonblock(int fd);
 extern void pool_unset_nonblock(int fd);
 extern void cancel_request(CancelPacket *sp);
+extern void check_stop_request(void);
 
 /* pool_process_query.c */
 void free_select_result(POOL_SELECT_RESULT *result);
 void reset_variables(void);
+void per_node_statement_log(POOL_CONNECTION_POOL *backend, int node_id, char *query);
 POOL_STATUS pool_extract_error_message(POOL_CONNECTION *backend, int major, bool unread, char **message);
 
 /* pool_relcache.c */
